/*
    This Software (C) Copyright David Goodwin, 2007, 2008.
    
     This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
        
/*************************************************************************
 * Test: DebugLoggerTest
 * Test Package: assignment
 * Purpose: To test the DebugLogger class
 * @author David Goodwin
 *************************************************************************/

package logging;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import junit.framework.TestCase;
import logging.ILogger.Severity;

/** Tests the Logger class.
 */
public class DebugLoggerTest extends TestCase {
    
    /** Log filename.
     */
    private final static String LOGFILE = "test.log";
    /** A message.
     */
    private final static String LFOFQABC = "Log file doesnt exist. Maybe open() failed?";
    
    /** A string.
     */
    private final static String STRING0001 = "Test";
    
    /** Another String.
     */
    private final static String STRING0002 = "Write should have succeded - file is OPEN";
    
    /** Constructor.
     * 
     * @param testName don't know
     */
    public DebugLoggerTest(final String testName) {
        super(testName);
    }            

    /** Setup unit tests. not currently used.
     */
    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    /** Tear down unit tests. Not currently used.
     */
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /** Tests the Open method.
     */
    public void testOpen() throws Exception {
        System.out.println("Testing Open method...");
        final String filename = LOGFILE;
        final DebugLogger instance = new DebugLogger();
        
        // We should be able to open a file
        try {
            instance.open(filename);
        } catch (final IOException e) {
            fail("An exception occured");
        }
        
        // We should NOT be able to open another file when we already have one
        // open
        try {
            instance.open("abacus");
            fail("An attempt to open a file while another was already open " +
                    "did not fail.");
        } catch (final IOException e) { 
            // do nothing
        }
        
        // Trash any created log file.
        killLogFile();
    }

    /** Tests the Close method.
     */
    public void testClose() throws Exception {
        System.out.println("Testing Close method...");
        final DebugLogger instance = new DebugLogger();
        final String filename = LOGFILE;
        
        // If no file is open, close() MUST throw an exception.
        try {
            instance.close();
            fail("No file was open - close should have thrown an exception");
        } catch (final IOException e) {  
            // do nothing
        }
        
        // Try to open a file. Assuming testOpen succeeded, this should work
        // fine. If this fails, then there is something wrong with testOpen.
        try {
            instance.open(filename);
        } catch (final IOException e) {
            fail("Something went wrong opening a file. testOpen likely " +
                    "failed too.");
        }
        
        // Try to close the just opened file. If this doesnt work then
        // something is wrong.
        try {
            instance.close();
        } catch (final IOException e) {
            fail("A file was open: The close should have worked fine");
        }
        
        // Try to close the file a second time. This should NOT succeed - it
        // was already closed.
        try {
            instance.close();
            fail("The file was already closed. A second attempt should have " +
                    "failed");
        } catch (final IOException e) { 
            // do nothing
        }
        
        // Trash any created log file
        killLogFile();
    }

    /** Tests writeLineE, writeLine(String) and writeLine(Severity,String). As
     * a side-effect it also test svToString(Severity).
     */
    public void testWriteLine() throws Exception {
        System.out.println("writeLine");
        DebugLogger instance = new DebugLogger();
        
        // 0: We should not be able to write when the log is closed
        try {
            instance.writeLine("Hello, World");
            instance.writeLineE(Severity.Warning, "The Abacus of Doom has escaped!");
            fail("Write should have failed: no file open");
        } catch (final IOException e) {
            // do nothing
        }
        // Close any possibly open logfile.
        try {
            instance.close();
        } catch(final IOException e){
            // do nothing
        }

        System.out.println("PASS: 0");
        
        // 1: Try to write a NOTICE severity message
        instance = openLogger();
        final String szSTR001325 = "This is a NOTICE";
        try {
            instance.writeLine(szSTR001325);
            instance.close();            
            findLine(true, "NOTICE: " + szSTR001325);
            killLogFile();
        } catch (final IOException e) { 
            fail(STRING0002); 
        }
        
        System.out.println("PASS: 1");
        
        // 2: Make sure the plain writeLineE function does the same as the
        //    one with severity level NOTICE
        tryLogWrite(Severity.Notice, "A notice");
        
        System.out.println("PASS: 2");
        
        // 3: Make sure DEBUG level messages are ARE printed
        tryLogWrite(Severity.Debug, "A Debug message");
        
        System.out.println("PASS: 3");
        
        // 4: Make sure that WARNING messages are printed
        tryLogWrite(Severity.Warning, "A Warning");
        System.out.println("PASS: 4");
        
        // 5: Make sure that ERROR messages are printed
        tryLogWrite(Severity.Error, "An Error");
        System.out.println("PASS: 5");
        
        // 6: Make sure that FAILURE messages are printed.
        tryLogWrite(Severity.Failure, "A Failure");
        System.out.println("PASS: 6");
        
        // 7: Try to print a message again. This should fail.
        try {
            instance.writeLineE(Severity.Notice, STRING0001);
            fail("Write should have failed - there is still no file open");
        } catch(final IOException e) {
            if (!e.getMessage().equals("No log file open")) {
                fail("Hmmn. Seems the exceptions user-helpful message is not so helpful.");
            }
        }
        System.out.println("PASS: 7");
        
        // 8: Try to print the message using writeLine(String). This should
        // pass as the exception is trapped there
        instance.writeLine(STRING0001);
        System.out.println("PASS: 8");        
        
        // 9: Try to print the message using writeLine(String). This should
        // pass as the exception is trapped there
        instance.writeLine(Severity.Notice, STRING0001);
        System.out.println("PASS: 9");
        
        // 10: Test writing of multiline messages:
        instance = openLogger();
        
        // First up, write a message to the log
        final String mumsg001 = "This is a\nMultiline Message";
        try {
            instance.writeLineE(Severity.Notice, mumsg001);
            instance.close();
        } catch(final IOException e) {
            fail("Open or Close failed. Something went wrong.");
        }
        
        // Then try and read it back
        final String[] lines = mumsg001.split("\n");
        int i = 0;
        boolean started = false;
        BufferedReader breader = null;
        try {
            final FileInputStream fin = new FileInputStream(LOGFILE);
            breader = new BufferedReader(new InputStreamReader(fin));            
        } catch (final IOException e) {
            fail(LFOFQABC);
        }
        
        try {
            String curlin = breader.readLine();
            while (curlin != null) {
                if (curlin.startsWith(lines[i])) {
                    i++;
                    started = true;
                } else if (started && i <= lines.length) {
                    fail("line \"" + lines[i] + "\" was not found at expected"+
                            "position in logfile");
                }
                curlin = breader.readLine();
            }
        } catch(final IOException e) { 
            if (i <= lines.length) {
                fail("There was an EOF before line \"" + lines[i] + "\" was found in logfile.");
            }
        }
        
        System.out.println("PASS: 10");
        
        System.out.println("writeLine passed");
    }
    
    /** Attempts to write a message to the log file and fails if it can be found
     * afterwards.
     * 
     * @param sv Severity Level
     * @param message Message to write
     */
    void tryLogWrite(final Severity sv, final String message) {
        final DebugLogger instance = openLogger();
        try {
            instance.writeLineE(sv, message);
            instance.close();
            findLine(true, instance.svToString(sv) + message);
        } catch (final IOException e) { 
            fail(STRING0002); 
        }
        killLogFile();
    }
    
    /** Opens the test log file.
     * @return A new instance of logger
     */
    DebugLogger openLogger() {
        final DebugLogger tl = new DebugLogger();
        try {
            tl.open(LOGFILE);
        } catch(final IOException e) {
            fail("Open is broken");
        }
        return tl;
    }
    
    /** Kills the created test log file.
     */
    void killLogFile() {
        final File f = new java.io.File(LOGFILE);
        f.delete();
    }
    
    /** Searches the log file for the specified line. If it can't be found then
     * it fails. If a multiline message was written, it will only check the
     * first line.
     * @param find If true it will fail if the line isn't present.
     * @param line The line to look for
     */ 
    void findLine(final boolean find, final String line) {
        BufferedReader breader = null;
        try {
            final FileInputStream fin = new FileInputStream(LOGFILE);
            breader = new BufferedReader(new InputStreamReader(fin));            
        } catch (final IOException e) {
            fail(LFOFQABC);
        }
        
        try {
            String curlin = breader.readLine();
            while (curlin != null) {
                if (curlin.startsWith(line)) {
                    return;
                }
                curlin = breader.readLine();
            }
        } catch(final IOException e) {
            if (find) {
                fail("EOF before line \"" + line + " was found in logfile."); 
            }
        }
        if (find) {
            fail("Line " + line + " not found in logfile.");
        }
    }
}
